package com.alterdekim.hearthhack.parser;

import com.alterdekim.hearthhack.dbf.CardsDBF;
import com.alterdekim.hearthhack.dbf.DBFCard;
import com.alterdekim.hearthhack.dbf.DBFColumn;
import com.alterdekim.hearthhack.dbf.DBFField;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

@Slf4j
public class DBFParser extends DefaultHandler {
    public static CardsDBF parseCards(String path) {
        SAXParserFactory factory = SAXParserFactory.newInstance();
        try (InputStream is = new FileInputStream(path + "/CARD.xml")) {
            SAXParser saxParser = factory.newSAXParser();
            DBFParser handler = new DBFParser();
            saxParser.parse(is, handler);
            return handler.getResult();
        } catch (ParserConfigurationException | SAXException | IOException e) {
            log.error(e.getMessage());
        }
        return new CardsDBF();
    }

    private final StringBuilder currentValue = new StringBuilder();

    @Getter
    private CardsDBF result;

    @Override
    public void startDocument() {
        result = new CardsDBF();
        result.setColumns(new ArrayList<>());
        result.setRecords(new ArrayList<>());
    }

    @Override
    public void startElement(
            String uri,
            String localName,
            String qName,
            Attributes attributes) {

        currentValue.setLength(0);

        switch (qName) {
            case "SourceFingerprint":
                result.setSourceFingerprint(attributes.getValue(qName));
                break;
            case "Column":
                DBFColumn column = new DBFColumn();
                column.setName(attributes.getValue("name"));
                column.setType(attributes.getValue("type"));
                result.getColumns().add(column);
                break;
            case "Record":
                result.getRecords().add(new DBFCard(new ArrayList<>()));
                break;
            case "Field":
                DBFField field = new DBFField();
                field.setColumn(attributes.getValue("column"));
                result.getRecords()
                        .get(result.getRecords().size()-1)
                        .getFields()
                        .add(field);
                break;
            case "Dbf":
                result.setName(attributes.getValue("name"));
        }
    }

    public void endElement(String uri,
                           String localName,
                           String qName) {
        if( qName.equals("Field") ) {
            List<DBFField> fields = result.getRecords()
                    .get(result.getRecords().size()-1)
                    .getFields();
            fields.get(fields.size()-1).setVal(currentValue.toString());
            result.getRecords()
                    .get(result.getRecords().size()-1)
                    .setFields(fields);
        }
    }

    public void characters(char[] ch, int start, int length) {
        currentValue.append(ch, start, length);
    }
}
